import { YamlService } from '../yaml-service';
import { UserRepository } from '../db/repositories/user-repository';
import knex from 'knex';
import config from '../db/knexfile';

const MIGRATIONS_DIR = config.test.migrations?.directory;

// Helper to truncate all tables
async function truncateAll(db: any) {
  await db.raw('PRAGMA foreign_keys = OFF');
  await db('notes').truncate();
  await db('steps').truncate();
  await db('tasks').truncate();
  await db('groups').truncate();
  await db('users').truncate();
  await db('images').truncate();
  await db('print_history').truncate();
  await db.raw('PRAGMA foreign_keys = ON');
}

describe('YamlService', () => {
  let db: ReturnType<typeof knex>;
  let yamlService: YamlService;

  beforeEach(async () => {
    db = knex(config.test);
    await db.migrate.latest({ directory: MIGRATIONS_DIR });
    // Create a user for notes
    const userRepo = new UserRepository(db);
    await userRepo.create({ name: 'Test User' });
    yamlService = new YamlService(db);
  });

  afterEach(async () => {
    await db.destroy();
  });

  it('should export empty database to YAML', async () => {
    const yamlContent = await yamlService.exportToYaml();
    expect(yamlContent).toContain('groups: []');
  });

  it('should import and export YAML correctly', async () => {
    const testYaml = `
groups:
  - name: Test Group
    tasks:
      - name: Test Task
        steps:
          - name: Test Step
            instructions: Test instructions
        notes:
          - content: Test note
    `;

    // Import the YAML
    await yamlService.importFromYaml(testYaml);

    // Export it back
    const exportedYaml = await yamlService.exportToYaml();

    // Should contain the imported data
    expect(exportedYaml).toContain('Test Group');
    expect(exportedYaml).toContain('Test Task');
    expect(exportedYaml).toContain('Test Step');
    expect(exportedYaml).toContain('Test instructions');
    expect(exportedYaml).toContain('Test note');
  });

  it('should handle invalid YAML', async () => {
    const invalidYaml = 'invalid: yaml: content:';

    await expect(yamlService.importFromYaml(invalidYaml)).rejects.toThrow();
  });
});